/******************************************************************************
 *                                                                            *
 *  Unit:         Config.h                                                    *
 *                                                                            *
 *  Copyright:    (c) Mikroelektronika, 2011.                                 *
 *                                                                            *
 *  Description:  Config.c declarations and some more.                        *
 *                                                                            *
 *  Requirements: PIC32 specific.                                             *
 *                                                                            *
 *  Migration:    Along with Config.c, this is the only file in this          *
 *                project that might need to be adjusted when migrating.      *
 *                Switching to another MCU within PIC24FJ family              *
 *                of microcontrollers, in most cases, will not require any    *
 *                change here :)                                              *
 *                                                                            *
 ****************************       CHANGE LOG       **************************
 * Version | ACTION                                           |  DATE  | SIG  *
 * --------|--------------------------------------------------|--------|----- *
 *         |                                                  |        |      *
 *    0.01 | - Initial release                                | 030511 |  ST  *
 *         |                                                  |        |      *
 ******************************************************************************/
#ifndef __CONFIG
#define __CONFIG

#include <Types.h>

extern const unsigned long MCU_FLASH_START_PHY;                   // Internal flash start address (physical).
extern const unsigned long MCU_BOOT_FLASH_START_PHY;              // Internal boot flash start address (physical).
extern const unsigned long MCU_BOOT_FLASH_CONFIG_BLOCK_START_PHY; // Boot flash row containing configuration words
                                                                  // start address (physical).

extern const unsigned long BOOTLOADER_SIZE;      // Bootloader size.
extern const unsigned long BOOTLOADER_START;     // Bootloader start virtual address.
extern const unsigned long BOOTLOADER_START_PHY; // Bootloader start physical address.
extern const unsigned char RESET_VECTOR_SIZE;    // MCU reset vector size.

extern const TBootInfo BootInfo;                 // Bootloader info record,
                                                 // containing device specific information.

extern unsigned char HidReadBuff[64];            // USB HID read buffer.
extern unsigned char HidWriteBuff[64];           // USB HID write buffer.

// Flash write and erase block sizes are MCU dependent.
// To reduce confusion and errors, these routines might not have
// uniform names between different MCUs/architectures.
// Consult library manager for target MCU's flash handling routine names.

// Some PIC32 MCU revisions had problem with flash block write
// on our side (possible silicon bug).
// That is why we will use single word write routines.
// Also, we'll need to override library _FLASH_WRITE_LATCH constant 
// to single word size.
// If your MCU does not suffer this issue, 
// just use FLASH_Write_Row define and
// loose _FLASH_WRITE_LATCH override.

#define FLASH_Write(addr, buff) FLASH_Write_Word(addr, *(long *)buff) // flash write one program word (4 bytes)
// #define FLASH_Write(addr, buff) FLASH_Write_Row(addr, buff)        // flash write whole row (512 bytes)

#define FLASH_Erase FLASH_Erase_Page             // flash erase (4096 bytes)

#define _FLASH_WRITE_LATCH 4                     // override to one program word (4 bytes)

void Config();                                   // multi purpose configuration routine
#endif